@@ -19,9 +19,199 @@
 # TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+import jinja2
+import os
+import re
+
+from majormode.perseus.model.locale import DEFAULT_LOCALE
+from majormode.perseus.model.locale import Locale
 from majormode.perseus.utils import string_util
 
 
+class EmailTemplate:
+    """
+    Template for generating the content of localized emails
+
+    An email can be localized in several languages (locales).  Each
+    localization corresponds to a file named after the locale in which
+    the content is written (RFC 4646).
+
+    A locale is expressed by a ISO 639-3 alpha-3 code element, optionally
+    followed by a dash character `-` and a ISO 3166-1 alpha-2 code.  For
+    example: "eng" (which denotes a standard English), "eng-US" (which
+    denotes an American English).
+
+    For instance:
+
+    /absolute/path/to/template/folder
+    ├── eng.txt
+    ├── fra.txt
+    └── vie.txt
+
+    This class uses Jinja template engine for rendering email content.
+    """
+    class NotRenderedError(Exception):
+        """
+        Signal that the email template has not been rendered with placeholders
+        """
+
+    def __init__(
+            self,
+            template_path: str,
+            locale: Locale = DEFAULT_LOCALE,
+            template_file_extension: str = '.txt'):
+        """
+        Create a new email template instance
+
+
+        :param template_path: The absolute path of the folder containing the
+            localized template files.
+
+        :param locale: The locale to generate email content.
+
+        :param template_file_extension: The extension of the localized
+            template files.
+        """
+        self.__template_path = template_path
+        self.__environment = jinja2.Environment(loader=jinja2.FileSystemLoader(self.__template_path))
+        self.__template = self.__environment.get_template(self.__get_template_file_name(locale))
+        self.__template_file_extension = template_file_extension
+        self.__content = None
+
+    def __get_template_file_name(self, locale: Locale = DEFAULT_LOCALE) -> str:
+        """
+        Return the absolute path and file name of the template for the
+        specified locale
+
+        If no template exists for the specified locale, the function returns
+        the absolute path and name of the template file for the default locale
+        `Locale.DEFAULT_LOCALE`.
+
+
+        :param locale: The locale of the desired template.
+
+
+        :return: The absolute path and file name of the template file for the
+            specified locale or the default locale
+        """
+        if not isinstance(locale, Locale):
+            raise ValueError("The argument `locale` must be an object `Locale`")
+
+        template_file_name = f'{locale.to_string()}{self.__template_file_extension}'
+        template_file_path_name = os.path.join(self.__template_path, template_file_name)
+
+        if not os.path.exists(template_file_path_name):
+            if locale == DEFAULT_LOCALE:
+                raise FileNotFoundError(f'No email template defined for the default locale "{DEFAULT_LOCALE}"')
+            return self.__get_template_file_name(DEFAULT_LOCALE)
+
+        return template_file_name
+
+    def render(self, **kwargs) -> str:
+        """
+        Render the content of the email in the specified locale
+
+
+        :param kwargs: Variables which values replace the corresponding
+            placeholders defined in the email content.
+
+
+        :return: The rendered template.
+        """
+        self.__content = self.__template.render(**kwargs)
+        return self.__content
+
+    @property
+    def content(self) -> str:
+        """
+        Return the content of the email that has been rendered
+
+
+        :return: The rendered content of the email.
+
+
+        :raise NotRenderedError: If the function `render` has not been called
+            yet.
+        """
+        if self.__content is None:
+            raise self.NotRenderedError("This email template must be rendered first")
+
+        return self.__content
+
+
+class EmailHtmlTemplate(EmailTemplate):
+    """
+    Template for generating the content of localized HTML emails
+
+    The subject of the email corresponds to the title of the HTML template
+    document.
+    """
+    class UndefinedTitleTagError(Exception):
+        """
+        Signal that the HTML document has not tag `title` defined.
+        """
+
+    REGEX_PATTERN_HTML_TITLE = r'<title>(.*)<\/title>'
+    REGEX_HTML_TITLE = re.compile(REGEX_PATTERN_HTML_TITLE)
+
+    def __init__(
+            self,
+            template_path: str,
+            locale: Locale = DEFAULT_LOCALE):
+        """
+        Create a new email template instance
+
+
+        :param template_path: The absolute path of the folder containing the
+            localized template files.
+
+        :param locale: The locale to generate email content.
+        """
+        super().__init__(template_path, locale=locale, template_file_extension=".html")
+        self.__subject = None
+
+    @classmethod
+    def __get_html_title(cls, content: str) -> str:
+        """
+        Return the title of the HTML document
+
+
+        :param content: The content of a HTML document.
+
+
+        :return: The value of the tag `title` of the HTMl document.
+
+
+        :raise UndefinedTitleTagError: If the HTML document doesn't contain
+            the HTML tag `title`.
+        """
+        match = cls.REGEX_HTML_TITLE.search(content)
+        if not match:
+            raise cls.UndefinedTitleTagError("The HTML content has no title defined")
+
+        return match.group(1).strip().capitalize()
+
+    @property
+    def subject(self) -> str:
+        """
+        Return the content of the email that has been rendered
+
+
+        :return: The rendered content of the email.
+
+
+        :raise NotRenderedError: If the function `render` has not been called
+            yet.
+
+        :raise UndefinedTitleTagError: If the HTML document doesn't contain
+            the HTML tag `title`.
+        """
+        if self.__subject is None:
+            self.__subject = self.__get_html_title(self.content)
+
+        return self.__subject
+
+
 class Mailbox:
     def __init__(self, email_address, name=None):
         """
